@@ -24,7 +24,9 @@ yatai_proto = LazyLoader('yatai_proto', globals(), 'bentoml.yatai.proto')
 @click.option('--path', help='directory path of pretrained model')
 @click.option('--labels', help='comma separated list of key:val pairs')
 @click.option('--files', help='comma separated list of file paths to add')
-def pack(module, clz, name, path, labels=None, files=None, opts=None):
+@click.option('--opts', help='comma separated list of key=val pairs')
+@click.option('--update', is_flag=True, help='update opts instead of replace')
+def pack(module, clz, name, path, labels=None, files=None, opts=None, update=False):
     # Create a service instance
     svc = get_instance(module, clz)
 
@@ -34,11 +36,11 @@ def pack(module, clz, name, path, labels=None, files=None, opts=None):
         raise click.BadParameter('length of names, paths does not match')
 
     for i, name in enumerate(names):
-        if opts is None:
-            opts = {}
+        if opts is not None:
+            opts = convert_labels(opts)
 
         # Package the pretrained model artifact
-        svc.pack(name, paths[i])
+        svc.pack(name, paths[i], opts=opts, update=update)
     
     if files is not None:
         filepaths = [x.strip() for x in files.split(',')]
@@ -61,16 +63,33 @@ def pack(module, clz, name, path, labels=None, files=None, opts=None):
     click.echo(saved_path)
 
 
+# The `__import__` function will return the top level module of a package, 
+# unless you pass a nonempty `fromlist` argument
+# https://stackoverflow.com/questions/9806963/how-to-use-the-import-function-to-import-a-name-from-a-submodule
 def get_instance(module_name, class_name):
-    module = __import__(module_name)
+    module = __import__(module_name, fromlist=['object'])
     class_ = getattr(module, class_name)
     return class_()
 
 
+def convert_value(val):
+    if val == 'True':
+        return True
+    elif val == 'False':
+        return False
+    # elif val.isdigit():
+    #     return int(val)
+    # try:
+    #     return float(val)
+    # except ValueError:
+    #     return val
+    return val
+
+
 def convert_labels(labels_str):
     pairs = [[y.strip() for y in x.strip().split('=')]
              for x in labels_str.split(',')]
-    return {p[0]:p[1] for p in pairs}
+    return {p[0]:convert_value(p[1]) for p in pairs}
 
 
 @click.command()
@@ -79,9 +98,11 @@ def convert_labels(labels_str):
 @click.option('--name', help='model name')
 @click.option('--bucket', help='bucket name of pretrained model', default='models')
 @click.option('--path', help='directory path of pretrained model')
-@click.option('--labels', help='comma separated list of key:val pairs')
+@click.option('--labels', help='comma separated list of key=val pairs')
 @click.option('--files', help='comma separated list of file paths to add')
-def pack_from_s3(module, clz, name, bucket, path, labels=None, files=None, opts=None):
+@click.option('--opts', help='comma separated list of key=val pairs')
+@click.option('--update', is_flag=True, help='update opts instead of replace')
+def pack_from_s3(module, clz, name, bucket, path, labels=None, files=None, opts=None, update=False):
     s3 = boto3.resource('s3',
         endpoint_url=os.environ['S3_ENDPOINT_URL'],
         aws_access_key_id=os.environ['AWS_ACCESS_KEY_ID'],
@@ -114,6 +135,8 @@ def pack_from_s3(module, clz, name, bucket, path, labels=None, files=None, opts=
 
     # Create a service instance
     svc = get_instance(module, clz)
+    if opts is not None:
+        opts = convert_labels(opts)
 
     for i, name in enumerate(names):
         tmp_path = '/tmp/{}/{}'.format(buckets[i], paths[i])
@@ -121,11 +144,8 @@ def pack_from_s3(module, clz, name, bucket, path, labels=None, files=None, opts=
         if not os.path.exists(tmp_path):
             download_directory(buckets[i], paths[i])
 
-        if opts is None:
-            opts = {}
-
         # Package the pretrained model artifact
-        svc.pack(name, tmp_path)
+        svc.pack(name, tmp_path, opts=opts, update=update)
 
     if files is not None:
         filepaths = [x.strip() for x in files.split(',')]
